﻿using System;
using System.Collections.Generic;
using System.Text;
using MarcusWS_Sample.DispatchManagerV1;
using System.Web.Services.Protocols;
namespace MarcusWS_Sample
{

    /// <summary>
    /// This class demonstrates using a Service Reference, as opposed to a Web Reference. One difference
    /// is that the Service Reference requires an InterfaceClient object instead of a Port object. Another difference is
    /// the change in the way that the request and header values are sent when the service is invoked.
    /// Service References are available with .NET Framework 3.5 and above.
    /// </summary>
    class DispatchManager
    {
        private static DispatchManagerV1InterfaceClient service;
        private String authenticationToken;

        public DispatchManager(String authenticationToken)
        {
            service = new DispatchManagerV1InterfaceClient();
            this.authenticationToken = authenticationToken;
        }

        public InVehicleMessage[] getMessagesByDate(DateTime since, DateTime end)
        {
            try
            {
                DispatchManagerV1.GetMessagesByDateRequest request = new DispatchManagerV1.GetMessagesByDateRequest();
                DWHeader header = getHeader(authenticationToken);
                request.MinDate = since;
                request.MaxDate = end;
                GetMessagesResponse response = service.getMessagesByDate(ref header, request);
                if (response != null)
                {
                    GetMessagesResponseInVehicleMessages messages = response.InVehicleMessages;
                    if (messages == null)
                        return null;
                    return messages.InVehicleMessage;
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return null;
        }

        public String createJob(string identifier, string identifierType, string jobName, Object location
            , string locationType, string message, DateTime startDate, TimeSpan jobDuration, DateTime sendDate)
        {
            try
            {
                DWHeader header = getHeader(authenticationToken);
                CreateJobRequest request = new CreateJobRequest();

                request.Identifier = identifier;
                request.IdentifierType = identifierType;
                request.JobName = jobName;
                request.LocationSource = locationType;
                request.JobLocation = location;
                request.MessageForDriver = message;
                request.JobDateTime = startDate;
                request.jobDuration = System.Xml.XmlConvert.ToString(jobDuration);
                request.SendDateTime = sendDate;
                request.SendDateTimeSpecified = true;

                CUDResponse response = service.createJob(ref header, request);
                if (response != null)
                {
                    String responseText = response.Result;
                    return responseText;
                }
                
            }catch (Exception ex){
                return "Error during request: " + ex; 
            }
            return "failed to get response";
        }

        private MarcusWS_Sample.DispatchManagerV1.DWHeader getHeader(String authenticationToken)
        {
            MarcusWS_Sample.DispatchManagerV1.DWHeader    header      = new MarcusWS_Sample.DispatchManagerV1.DWHeader();
            MarcusWS_Sample.DispatchManagerV1.SenderInfo  senderInfo  = new MarcusWS_Sample.DispatchManagerV1.SenderInfo();
            MarcusWS_Sample.DispatchManagerV1.Credentials credentials = new MarcusWS_Sample.DispatchManagerV1.Credentials();
            senderInfo.ApplicationID = "SampleClient2"; //user supplied value for tracking

            header.SenderInfo = senderInfo;
            senderInfo.Credentials = credentials;
            credentials.AuthToken = authenticationToken;

            return header;
        }
    }
}