﻿using System;
using System.Collections.Generic;
using System.Text;
using MarcusWS_Sample.DriverManagerV1;

namespace MarcusWS_Sample
{
    public class DriverManager
    {
        private String authenticationToken;
        private static DriverManagerV1Port service = new DriverManagerV1Port();

        public DriverManager(String authenticationToken)
        {
            this.authenticationToken = authenticationToken;
            service.DWHeaderValue = GetDriverManagerHeader();
        }

        private DWHeader GetDriverManagerHeader()
        {
            DWHeader header = new DWHeader();
            SenderInfo si = new SenderInfo();
            si.ApplicationID = "SampleClient2"; //user supplied value for tracking

            Credentials cred = new Credentials();
            cred.AuthToken = authenticationToken;
            si.Credentials = cred;
            header.SenderInfo = si;

            return header;
        }

        /// <summary>
        /// Get driver details filtered by given criteria.
        /// </summary>
        /// <param name="criteria"></param>
        /// <param name="sortby"></param>
        /// <param name="isAscending"></param>
        /// <returns></returns>
        public List<DriverPlusAttributes> GetDriverDetails(Dictionary<string, string> filters, string sortby, bool isAscending, int pageMax, int pageNumber)
        {
            try
            {
                GetDriverDetailsRequest request = new GetDriverDetailsRequest();

                request.MaxResults = pageMax;
                request.MaxResultsSpecified = true;  //must be set, otherwise MaxResults is not sent to server.
                request.Page = pageNumber;
                request.ascending = isAscending;

                if (sortby == null)
                {
                    request.SortBy = "lastname";
                }
                else
                {
                    request.SortBy = sortby;
                }

                if (filters != null && filters.Count > 0)
                {
                    List<FilterCriteria> filterCriteria = new List<FilterCriteria>();
                    
                    //Iterate over all filter criteria and add to the request obj.
                    //There is no limit to the number of filters that can be applied.
                    foreach (KeyValuePair<string,string> kv in filters)
                    {
                        if (kv.Value == null || kv.Value.Trim().Equals("")) continue;

                        FilterCriteria criteria = new FilterCriteria();
                        criteria.FilterKey = kv.Key;
                        criteria.SearchTerm = kv.Value;
                        filterCriteria.Add(criteria);
                    }

                    if (filterCriteria.Count > 0)
                    {
                        request.FilterCriteria = filterCriteria.ToArray();
                    }
                }

                GetDriverDetailsResponse allDrivers = service.getDriverDetails(request);

                if (allDrivers.Driver == null) return null;

                return new List<DriverPlusAttributes>( allDrivers.Driver);
            }
            catch (Exception ex)
            {
                Console.Write("Error: " + ex);
                return null;
            }
        }
    }
}
