using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Web.Services.Protocols;
using System.Windows.Forms;
using MarcusWS_Sample.AlertManagerV2;
using MarcusWS_Sample.MobileManagerV1;
using MarcusWS_Sample.TrackManagerV1;
using MarcusWS_Sample.DispatchManagerV1;

namespace MarcusWS_Sample
{
    public partial class FormMain : Form
    {
        bool isInitDone = false;

        public FormMain()
        {
            InitializeComponent();
            comboBoxAMtype.SelectedIndex = 0;
            comboBoxAMactiveTimes.SelectedIndex = 0;
            comboBoxAMappliesTo.SelectedIndex = 0;
            comboBoxAMactiveDays.SelectedIndex = 0;
            comboBoxAMactiveTimes.SelectedIndex = 0;
            jobLocationTypecbox.SelectedIndex = 0;
            identifierTypecbox.SelectedIndex = 0;

            //get driver details settings
            List<String> filterCriteria = new List<string>();
            String[] filters = { "lastName", "firstName", "driverId", "assignedMobileHardwareId"
                                   , "stateDL", "otherDL", "keyFobId", "assignedMobileName", "driverPhoneNumber" };
            filterCriteria.AddRange(filters);

            filterCriteriaComboBox.DataSource = getFilterCriteria();
            filterCriteriaComboBox.SelectedIndex = 0;
            filterCriteriaComboBox.SelectedValueChanged += new EventHandler(filterCriteriaComboBox_SelectedIndexChanged);
            filterCriteriaComboBox2.DataSource = getFilterCriteria();
            filterCriteriaComboBox2.SelectedIndex = 1;
            filterCriteriaComboBox2.SelectedValueChanged += new EventHandler(filterCriteriaComboBox_SelectedIndexChanged);
            sortByComboBox.DataSource = getFilterCriteria();
            sortByComboBox.SelectedIndex = 0;
            ascendingComboBox.SelectedIndex = 0;

            resultsPerPageInput.Validating += new CancelEventHandler(Input_Validating);
            pageNumberInput.Validating += new CancelEventHandler(Input_Validating);

            // keep the following the last line.
            isInitDone = true;
        }

        /// <summary>
        /// each combo box needs a seperate instance of the datasource.
        /// </summary>
        /// <returns></returns>
        private List<string> getFilterCriteria()
        {
            List<String> filterCriteria = new List<string>();
            String[] filters = { "lastName", "firstName", "driverId", "assignedMobileHardwareId"
                                   , "stateDL", "otherDL", "keyFobId", "assignedMobileName", "driverPhoneNumber" };
            filterCriteria.AddRange(filters);

            return filterCriteria;
        }

        //get authentication token from DW using username and password from form
        //store token in AuthenticationToken textboxes on various tabs.
        private void btnGetToken_Click(object sender, EventArgs e)
        {
            try
            {
                String token = WebServices.GetToken(txtUsername.Text, txtPassword.Text);
                if (token == null)
                {
                    throw new SoapException("Token not obtained.",null);
                }
                MessageBox.Show("Token successfully obtained.");
                txtToken.Text = token;
                txtTokenForGetMessages.Text = token;
                sendJobToMobileAuthenticationTokenText.Text = token;
                driverDetailsAuthToken.Text = token;
                btnGetTracks.Enabled = true;
                sendJobButton.Enabled = true;
            }
            catch (SoapException ex)
            {
                MessageBox.Show(Utils.GetErrorMessage(ex));
            }

        }

        private void btnClearToken_Click(object sender, EventArgs e)
        {
            txtToken.Text = "";
            btnGetTracks.Enabled = false;
        }

        //sample method to retrieve tracks from DW TrackManagerV1 webService
        //write track data to text file for proof of concept
        private void btnGetTracks_Click(object sender, EventArgs e)
        {
            if (txtMobileName.Text.Equals(""))
            {
                MessageBox.Show("You must enter a mobile name before getting tracks.");
                return;
            }
            try
            {
                String m = txtMobileName.Text;
                SoapHeader header = Utils.GetTrackManagerHeader(txtToken.Text);
                MarcusWS_Sample.TrackManagerV1.Track[] tracks = WebServices.GetTracksForMobile(header, m, dateTimePicker.Value);
                int numtracks = 0;
                StringBuilder sb = new StringBuilder();
                if (tracks != null)
                {
                    sb.Append("mobilename,timestamp,lat,lon,street,city,state,zip,motion,ignition,heading").Append(Environment.NewLine);
                    foreach (MarcusWS_Sample.TrackManagerV1.Track t in tracks)
                    {
                        sb.Append(t.MobileName).Append(",").Append(t.TimeStamp).Append(",").
                        Append(t.Lat).Append(",").Append(t.Lon).Append(",").
                        Append(t.Street).Append(",").Append(t.City).Append(",").
                        Append(t.StateProvince).Append(",").Append(t.PostalCode).Append(",").
                        Append(t.Motion).Append(",").Append(t.Ignition).Append(",").
                        Append(t.Heading).Append(Environment.NewLine);
                    }
                    Utils.WriteTracksToFile(txtFileName.Text, sb.ToString() + Environment.NewLine);
                    numtracks = tracks.Length;
                }
                MessageBox.Show("There were " + numtracks + " tracks written to file '" + txtFileName.Text + "'.");
            }
            catch (SoapException ex)
            {
                MessageBox.Show(Utils.GetErrorMessage(ex));
            }              
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            btnGetMessages.Enabled = true;
        }

        private void txtToken_LostFocus(object sender, EventArgs e)
        {
            if (txtToken.Text.Trim().Length > 0)
            {
                btnGetTracks.Enabled = true;
            }

            if (txtTokenForGetMessages.Text.Trim().Length > 0)
            {
                btnGetMessages.Enabled = true;
            }
        }

        private void btnGetMessages_Click(object sender, EventArgs e)
        {
            try
            {
                InVehicleMessage[] messages = new DispatchManager(txtTokenForGetMessages.Text).getMessagesByDate(dateTimePickerBegin.Value, dateTimePickerEnd.Value);
                int numMessages = 0;
                StringBuilder sb = new StringBuilder();
                if (messages != null)
                {
                    sb.Append("From, To, Message, Type, Reply, SentDate, DeliveryStatus").Append(Environment.NewLine);
                    foreach (InVehicleMessage msg in messages)
                    {
                        sb.Append(msg.From).Append(",").Append(msg.To).Append(",").
                        Append(msg.Message).Append(",").Append(msg.Type).Append(",").
                        Append(msg.Reply).Append(",").Append(msg.SentDate).Append(",").
                        Append(msg.DeliveryStatus).Append(Environment.NewLine);
                    }
                    Utils.WriteTracksToFile(txtMessageFileLocation.Text, sb.ToString() + Environment.NewLine);
                    numMessages = messages.Length;
                }
                MessageBox.Show("There were " + numMessages + " messages written to file '" + txtMessageFileLocation.Text + "'.");
            }
            catch (Exception ex)
            {
                MessageBox.Show(Utils.GetErrorMessage(ex));
            }
        }

        private void btnCreateAlertDef_Click(object sender, EventArgs e)
        {
            try
            {
                //This should be modified to different Request, if UI type is set to something different from 'Arrive Location'.
                MarcusWS_Sample.AlertManagerV2.CreateAlertArriveLocationRequest messagePart = new MarcusWS_Sample.AlertManagerV2.CreateAlertArriveLocationRequest(); 
                messagePart.Name = textBoxAMname.Text;
                if (checkBoxAMenabled.Enabled == true)
                    messagePart.Enabled = true;
                else
                    messagePart.Enabled = false;

                MarcusWS_Sample.AlertManagerV2.AppliesTo appliesTo = new MarcusWS_Sample.AlertManagerV2.AppliesTo();
                switch (comboBoxAMappliesTo.SelectedIndex)
                {
                    case 0: //allMobiles
                        appliesTo.Item = MarcusWS_Sample.AlertManagerV2.AllMobiles.AllMobiles;
                        break;
                    case 1: //selectedMobiles
                        String mobiles = textBoxAMselected.Text;
                        MarcusWS_Sample.AlertManagerV2.StringList hardwareIdList = new MarcusWS_Sample.AlertManagerV2.StringList();
                        hardwareIdList.Entry = mobiles.Split(new char[] { '\n', '\r', ' ', ';' }, StringSplitOptions.RemoveEmptyEntries);
                        appliesTo.Item = hardwareIdList;
                        appliesTo.ItemElementName = MarcusWS_Sample.AlertManagerV2.ItemChoiceType.SelectedMobiles;
                        break;
                    case 2: //selectedFleets
                        String fleets = textBoxAMselected.Text;
                        MarcusWS_Sample.AlertManagerV2.StringList fleetIdList = new MarcusWS_Sample.AlertManagerV2.StringList();
                        fleetIdList.Entry = fleets.Split(new char[] { '\n', '\r', ' ', ';' }, StringSplitOptions.RemoveEmptyEntries);
                        appliesTo.Item = fleetIdList;
                        appliesTo.ItemElementName = MarcusWS_Sample.AlertManagerV2.ItemChoiceType.SelectedFleets;
                        break;
                }
                messagePart.AppliesTo = appliesTo;

                MarcusWS_Sample.AlertManagerV2.ActiveDays activeDays = new MarcusWS_Sample.AlertManagerV2.ActiveDays();
                switch (comboBoxAMactiveDays.SelectedIndex)
                {
                    case 0:
                        activeDays.Item = MarcusWS_Sample.AlertManagerV2.ActiveDaysChoices.@Item7daysPerWeek;
                        break;
                    case 1:
                        activeDays.Item = MarcusWS_Sample.AlertManagerV2.ActiveDaysChoices.@Weekdays;
                        break;
                    case 2:
                        activeDays.Item = MarcusWS_Sample.AlertManagerV2.ActiveDaysChoices.@Weekends;
                        break;
                    case 3:
                        List<MarcusWS_Sample.AlertManagerV2.WeekDayChoices> dayList = new List<MarcusWS_Sample.AlertManagerV2.WeekDayChoices>();
                        if (checkBoxAMsu.Checked == true) dayList.Add(MarcusWS_Sample.AlertManagerV2.WeekDayChoices.Su);
                        if (checkBoxAMmo.Checked == true) dayList.Add(MarcusWS_Sample.AlertManagerV2.WeekDayChoices.Mo);
                        if (checkBoxAMtu.Checked == true) dayList.Add(MarcusWS_Sample.AlertManagerV2.WeekDayChoices.Tu);
                        if (checkBoxAMwe.Checked == true) dayList.Add(MarcusWS_Sample.AlertManagerV2.WeekDayChoices.We);
                        if (checkBoxAMth.Checked == true) dayList.Add(MarcusWS_Sample.AlertManagerV2.WeekDayChoices.Th);
                        if (checkBoxAMfr.Checked == true) dayList.Add(MarcusWS_Sample.AlertManagerV2.WeekDayChoices.Fr);
                        if (checkBoxAMsa.Checked == true) dayList.Add(MarcusWS_Sample.AlertManagerV2.WeekDayChoices.Sa);
                        MarcusWS_Sample.AlertManagerV2.CustomDays customDays = new MarcusWS_Sample.AlertManagerV2.CustomDays();
                        customDays.WeekdayList = dayList.ToArray();
                        activeDays.Item = customDays;
                        break;
                }
                messagePart.ActiveDays = activeDays;

                //This should be modified to different Request, if UI type is set to something different from 'Arrive Location'.
                MarcusWS_Sample.AlertManagerV2.AllOrBetweenTimes allOrBetweenTimes = new MarcusWS_Sample.AlertManagerV2.AllOrBetweenTimes();
                allOrBetweenTimes.Item = new MarcusWS_Sample.AlertManagerV2.TwentyFourHoursPerDay();
                messagePart.AllOrBetweenTimes = allOrBetweenTimes;

                MarcusWS_Sample.AlertManagerV2.StringList stringList = new MarcusWS_Sample.AlertManagerV2.StringList();
                String email = textBoxAMemails.Text;
                stringList.Entry = email.Split(new char[] { '\n', '\r', ' ', ';' }, StringSplitOptions.RemoveEmptyEntries);
                messagePart.Emails = stringList;
                
                messagePart.UserId = textBoxAMuserId.Text;

                messagePart.ArriveLocation = "500";

                //This should be modified to different Request, if UI type is set to something different from 'Arrive Location'.
                int result = AlertManager.CreateAlertArriveLocation(txtAuthTokenAlertManager.Text, messagePart);
                MessageBox.Show("Create alert definition return " + result);
            }
            catch (SoapException ex)
            {
                MessageBox.Show(Utils.GetErrorMessage(ex));
            }
        }

        private void button7_Click(object sender, EventArgs e)
        {
            txtAuthTokenAlertManager.Text = "";
            button7.Enabled = false;
        }

        private void button6_Click(object sender, EventArgs e)
        {
            txtTokenForGetMessages.Text = "";
            button6.Enabled = false;
        }

        private void txtAuthTokenAlertManager_TextChanged(object sender, EventArgs e)
        {
            if (button7.Enabled == false)
                button7.Enabled = true;
        }

        private void txtTokenForGetMessages_TextChanged(object sender, EventArgs e)
        {
            if (button6.Enabled == false)
                button6.Enabled = true;
        }

        private void comboBoxAMactiveDays_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBoxAMactiveDays.SelectedIndex == 0) //7 dyas
            {
                checkBoxAMsu.Checked = checkBoxAMmo.Checked = checkBoxAMtu.Checked =
                    checkBoxAMwe.Checked = checkBoxAMth.Checked = checkBoxAMfr.Checked =
                    checkBoxAMsa.Checked = true;
            }
            else if (comboBoxAMactiveDays.SelectedIndex == 1)  // weekdays
            {
                checkBoxAMmo.Checked = checkBoxAMtu.Checked = checkBoxAMwe.Checked = 
                    checkBoxAMth.Checked = checkBoxAMfr.Checked = true;
                checkBoxAMsu.Checked = checkBoxAMsa.Checked = false;
            }
            else if (comboBoxAMactiveDays.SelectedIndex == 2)    //weekends
            {
                checkBoxAMmo.Checked = checkBoxAMtu.Checked = checkBoxAMwe.Checked =
                    checkBoxAMth.Checked = checkBoxAMfr.Checked = false;
                checkBoxAMsu.Checked = checkBoxAMsa.Checked = true;
            }
        }

        private void checkBoxAMday_CheckedChanged(object sender, EventArgs e)
        {
            if (!isInitDone) return;
            if (comboBoxAMactiveDays.SelectedIndex != 3) comboBoxAMactiveDays.SelectedIndex = 3;
        }

        private void comboBoxAMactiveTimes_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBoxAMactiveTimes.SelectedIndex == 0) //24 hours
            {
                comboBoxAMstart.Visible = labelAMbetween.Visible = comboBoxAMend.Visible = false;
            }
            else if (comboBoxAMactiveTimes.SelectedIndex == 1)  // between
            {
                comboBoxAMstart.Visible = labelAMbetween.Visible = comboBoxAMend.Visible = true;
            }
        }

        private void comboBoxAMtype_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBoxAMtype.SelectedIndex == 0) //Arrive Location
            {
                labelAMparamName.Visible = textBoxAMparam.Visible = true;
                labelAMparamName.Text = "Selected Location:";
            }
            else if (comboBoxAMtype.SelectedIndex == 1) //Depart Location
            {
            }
            else if (comboBoxAMtype.SelectedIndex == 2) //Enter Zone
            {
            }
            else if (comboBoxAMtype.SelectedIndex == 3) //Exit Zone
            {
            }
            else if (comboBoxAMtype.SelectedIndex == 4) //First Movement
            {
            }
            else if (comboBoxAMtype.SelectedIndex == 5) //No Movement
            {
            }
            else if (comboBoxAMtype.SelectedIndex == 6) //Motion
            {
            }
            else if (comboBoxAMtype.SelectedIndex == 7) //Extended Stop
            {
            }
            else if (comboBoxAMtype.SelectedIndex == 8) //Speed
            {
            }
            else if (comboBoxAMtype.SelectedIndex == 9) //Posted Speed
            {
            }
            else if (comboBoxAMtype.SelectedIndex == 10) //Sensor Active
            {
            }
            else if (comboBoxAMtype.SelectedIndex == 11) //Sensor Inactive
            {
            }
            else if (comboBoxAMtype.SelectedIndex == 12) //Sensor Act/Inact
            {
            }
            else if (comboBoxAMtype.SelectedIndex == 13) //Enter Zip Code
            {
            }
            else if (comboBoxAMtype.SelectedIndex == 14) //Exit Zip Code
            {
            }
            else if (comboBoxAMtype.SelectedIndex == 15) //Idle
            {
            }
            else if (comboBoxAMtype.SelectedIndex == 16) //High Temperature
            {
            }
        }

        private void comboBoxAMappliesTo_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBoxAMappliesTo.SelectedIndex == 0) //all mobiles
            {
                labelAMselected.Visible = textBoxAMselected.Visible = false;
            }
            else if (comboBoxAMappliesTo.SelectedIndex == 1)  // selected mobiles
            {
                labelAMselected.Visible = textBoxAMselected.Visible = true;
                labelAMselected.Text = "Selected Mobiles";
            }
            else // selected fleet
            {
                labelAMselected.Visible = textBoxAMselected.Visible = true;
                labelAMselected.Text = "Selected Fleets";
            }
        }

        private void sendJobButton_Click(object sender, EventArgs e)
        {
            try
            {
                if (isEmpty(identifierTypeInput.Text, "Identifier"))
                {
                    return;
                }
                if (isEmpty(jobNameText.Text,    "job name"))
                {
                    return;
                }
                if (isEmpty(jobLocationTypeInput.Text, "location id"))
                {
                    return;
                }
                if (isEmpty(messageText.Text,    "message"))
                {
                    return;
                }

                object location = null;
                if (jobLocationTypecbox.SelectedIndex == 0) //lat,lon
                {
                    MarcusWS_Sample.DispatchManagerV1.Point point = new MarcusWS_Sample.DispatchManagerV1.Point();
                    string[] loc = jobLocationTypeInput.Text.Split(',');
                    if (loc.Length < 2)
                    {
                        return;
                    }
                    point.x = Convert.ToDouble(loc[0]);
                    point.y = Convert.ToDouble(loc[1]);
                    location = point;
                }
                else
                {
                    //location id
                    location = jobLocationTypeInput.Text;
                }

                TimeSpan jobDuration = new TimeSpan( Convert.ToInt32(hourDuration.Text), 0, 0);

                string locationType = "";
                if (jobLocationTypecbox.SelectedIndex == 0)
                {
                    locationType = "lat_lon";
                }
                else if(jobLocationTypecbox.SelectedIndex == 1)
                {
                    locationType = "location";
                }

                DispatchManager dispatchManager = new DispatchManager(sendJobToMobileAuthenticationTokenText.Text);

                String response = dispatchManager.createJob(identifierTypeInput.Text, (string)identifierTypecbox.SelectedItem, jobNameText.Text, location, locationType, messageText.Text, JobDateTimeInput.Value, jobDuration, jobSendDateInput.Value);
                MessageBox.Show("The job " + jobNameText.Text + " sent a message and the response was " + response);
            }
            catch (SoapException ex)
            {
                MessageBox.Show(Utils.GetErrorMessage(ex));
            }
        }

        private bool isEmpty(String text, string errorType)
        {
            if (text.Equals(""))
            {
                MessageBox.Show("You must enter a " + errorType + " before sending a Job.");
                return true;
            }
            return false;
        }

        /// <summary>
        /// handle user requesting driver details
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void driverDetailsBtn_Click(object sender, EventArgs e)
        {
            DriverManager driverManager = new DriverManager(driverDetailsAuthToken.Text);

            Dictionary<string, string> filters = new Dictionary<string, string>();
            filters.Add(filterCriteriaComboBox.SelectedItem.ToString(), filterCriteria1.Text);
            filters.Add(filterCriteriaComboBox2.SelectedItem.ToString(), filterCriteria2.Text);

            int pageMax = Convert.ToInt32(resultsPerPageInput.Text);
            int pageNumber = Convert.ToInt32(pageNumberInput.Text);

            List<MarcusWS_Sample.DriverManagerV1.DriverPlusAttributes> drivers = driverManager.GetDriverDetails(filters, sortByComboBox.SelectedItem.ToString(), Convert.ToBoolean(ascendingComboBox.SelectedItem.ToString()), pageMax, pageNumber);
            int totalDrivers = 0;
            if (drivers != null && drivers.Count > 0)
            {
                StringBuilder sb = new StringBuilder();

                sb.Append("FirstName, LastName, EmailAddress, PhoneNumber, DriverId, AdditionalFields:key,AdditionalFields:value").Append(Environment.NewLine);
                foreach (MarcusWS_Sample.DriverManagerV1.DriverPlusAttributes d in drivers)
                    {
                        sb.Append(d.FirstName).Append(",").Append(d.LastName).Append(",").
                        Append(d.EmailAddress).Append(",").Append(d.PhoneNumber).Append(",").Append(d.DriverId).Append(",");

                        if (d.Attributes != null)
                        {
                            foreach (MarcusWS_Sample.DriverManagerV1.AdditionalAttributes attribute in d.Attributes)
                            {
                                if (attribute.key.Equals("keyFobId") || attribute.key.Equals("stateDl") || attribute.key.Equals("otherDl"))
                                {
                                    sb.Append(" key: " + attribute.key + " value:" + attribute.StringValue);
                                }
                                if (attribute.key.Equals("assignedMobile"))
                                {
                                    MarcusWS_Sample.DriverManagerV1.MobileBrief mb = (MarcusWS_Sample.DriverManagerV1.MobileBrief)attribute.ObjectValue;
                                    sb.Append(" key: " + attribute.key + " value: " + mb.HardwareId + " " + mb.Name);
                                }

                            }
                        }

                        sb.Append(Environment.NewLine);
                    }
                    Utils.WriteTracksToFile(driverDetailsTxtFile.Text, sb.ToString() + Environment.NewLine);
                    totalDrivers = drivers.Count;
             }

            MessageBox.Show("There were " + totalDrivers + " drivers written to file '" + driverDetailsTxtFile.Text + "'.");
        }

        /// <summary>
        /// make sure that the same filter criteria is not sent twice.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void filterCriteriaComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            
            if (filterCriteriaComboBox2.SelectedIndex == filterCriteriaComboBox.SelectedIndex)
            {
                ComboBox cb = (ComboBox)sender;

                if (cb.SelectedIndex >= cb.Items.Count - 1)
                {
                    cb.SelectedIndex = 0;
                }
                else
                {
                    cb.SelectedIndex = cb.SelectedIndex + 1;
                }
            }
        }

        private void Input_Validating(object sender, EventArgs e)
        {
            try
            {
                int numText = Convert.ToInt32(((TextBox)sender).Text);
            }
            catch (Exception)
            {
                MessageBox.Show("Invalid entry, must be an integer.");
                ((TextBox)sender).Text = "0";
            }
        }

        private void jobLocationTypecbox_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox cb = (ComboBox)sender;
            if (cb.SelectedIndex == 0) //lat/lon
            {
                jobLocationTypeInput.Text = "41.973552,-87.73484";
            }
            else if(cb.SelectedIndex == 1) //locationId
            {
                jobLocationTypeInput.Text = "";
            }
        }


        private void jobDurationInput_Scroll(object sender, EventArgs e)
        {
            TrackBar tb = (TrackBar)sender;
            hourDuration.Text = Convert.ToString(tb.Value );
        }

        private void identifierTypecbox_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox cb = (ComboBox)sender;
            if (cb.SelectedIndex == 0 || cb.SelectedIndex == 1) //hardware id or userid
            {
                IdentifierTypeLabel.Visible = true;
                identifierTypeInput.Visible = true;
                identifierTypeInput.Text = "";
            }

            else if (cb.SelectedIndex == 2) //unassigned
            {
                IdentifierTypeLabel.Visible = false;
                identifierTypeInput.Visible = false;
                identifierTypeInput.Text = "Unassigned";
            }
        }
    }
}